"""Main module for LG3K - Log Generator 3000."""

import importlib
import json
import os
import shutil
import sys
import threading
import time
from datetime import datetime
from pathlib import Path
from types import SimpleNamespace
from typing import Callable, Dict, Optional, Union

import click
from rich.console import Console
from rich.panel import Panel
from rich.table import Table

# Try to import Rich, but don't fail if it's not available
try:
    console = Console()
    HAS_RICH = True
except ImportError:
    console = None
    HAS_RICH = False

from .utils.config import get_default_config, load_config

__version__ = "0.6.7"

# Global lock for progress updates
progress_lock = threading.Lock()
# Global progress state
module_progress = {}
# Track module status
module_status = {}
# Track module order
module_order = []
# Global state for exit handling
exit_event = threading.Event()
current_run_files = set()


def get_terminal_width() -> int:
    """Get the terminal width, defaulting to 80 if not available."""
    try:
        return shutil.get_terminal_size().columns
    except (AttributeError, OSError):
        return 80


def show_rich_help(ctx: click.Context) -> None:
    """Show help message with Rich formatting.

    Args:
        ctx: Click context object
    """
    if not HAS_RICH:
        click.echo(
            "Rich library not available. Install 'rich' package for enhanced output."
        )
        click.echo(ctx.get_help())
        return

    try:
        # Create table for options
        table = Table(
            title="Command Line Options", show_header=True, header_style="bold magenta"
        )
        table.add_column("Option", style="cyan", no_wrap=True)
        table.add_column("Description", style="green")
        table.add_column("Default", style="yellow")

        # Add options to table
        for param in ctx.command.params:
            default = param.default if param.default != param.required else ""
            if isinstance(default, bool):
                default = str(default).lower()
            elif default is None:
                default = ""
            table.add_row(
                f"--{param.name}",
                param.help or "",
                str(default),
            )

        # Create panel for description
        description = Panel(
            "Multi-threaded log generator for testing and development.\n\n"
            "Start with: lg3k --generate-config config.json\n"
            "Press Ctrl+C to exit gracefully.",
            title="Description",
            border_style="blue",
        )

        # Print help message
        console.print(description)
        console.print(table)

    except Exception as e:
        click.echo(f"Error displaying Rich help: {str(e)}")
        click.echo(ctx.get_help())


def load_modules() -> Dict[str, callable]:
    """Load all log generation modules."""
    modules = {}
    module_dir = os.path.join(os.path.dirname(__file__), "modules")

    for file in os.listdir(module_dir):
        if file.endswith(".py") and not file.startswith("__"):
            module_name = file[:-3]
            try:
                module = importlib.import_module(
                    f".modules.{module_name}", package="lg3k"
                )
                if hasattr(module, "generate_log"):
                    modules[module_name] = module.generate_log
            except ImportError as e:
                if HAS_RICH:
                    console.print(
                        f"[yellow]Warning: Failed to load module {module_name}: {e}[/yellow]"
                    )
                else:
                    print(f"Warning: Failed to load module {module_name}: {e}")

    return modules


def format_progress_display() -> str:
    """Format progress display with each module on its own line."""
    lines = []
    # Use module_order instead of sorting by name
    for name in module_order:
        status = module_status.get(name, "Waiting")
        # Add hash to module name for Docker-like display
        module_id = f"{hash(name) & 0xFFFFFFFF:08x}"
        if status == "Running":
            progress = module_progress.get(name, "0%")
            if HAS_RICH and console is not None:
                lines.append(
                    f"[cyan]{module_id}[/cyan]: [green]{name:<12}[/green] {progress}"
                )
            else:
                lines.append(f"{module_id}: {name:<12} {progress}")
        elif status == "Complete":
            if HAS_RICH and console is not None:
                lines.append(
                    f"[cyan]{module_id}[/cyan]: [green]{name:<12} Complete[/green]"
                )
            else:
                lines.append(f"{module_id}: {name:<12} Complete")
        elif status.startswith("Error:"):
            if HAS_RICH and console is not None:
                lines.append(
                    f"[cyan]{module_id}[/cyan]: [red]{name:<12} {status}[/red]"
                )
            else:
                lines.append(f"{module_id}: {name:<12} {status}")
        else:
            if HAS_RICH and console is not None:
                lines.append(
                    f"[cyan]{module_id}[/cyan]: [yellow]{name:<12} {status}[/yellow]"
                )
            else:
                lines.append(f"{module_id}: {name:<12} {status}")
    return "\n".join(lines)


def update_progress_display():
    """Update the progress display."""
    if not module_progress:
        return

    # Move cursor up by number of modules
    module_count = len(module_progress)
    if module_count > 0:
        try:
            # Clear previous display
            print(f"\033[{module_count}A\033[J", end="", flush=True)
            # Print new display
            if HAS_RICH and console is not None:
                try:
                    console.print(format_progress_display())
                except Exception:
                    print(format_progress_display(), flush=True)
            else:
                print(format_progress_display(), flush=True)
        except Exception:
            # If terminal control fails, just print the current status
            print(format_progress_display(), flush=True)


def cleanup_files(keep_files: bool = False):
    """Clean up generated files from the current run.

    Args:
        keep_files: If True, keep the partially generated files
    """
    if not keep_files:
        # Convert to list to avoid modifying set during iteration
        files_to_remove = list(current_run_files)
        for file_path in files_to_remove:
            try:
                Path(file_path).unlink()
                current_run_files.remove(file_path)
            except OSError:
                pass  # Ignore errors during cleanup


def generate_analysis(name: str, log_entry: Dict) -> str:
    """Generate analysis for a log entry based on its type and content.

    Args:
        name: Module name
        log_entry: The log entry to analyze

    Returns:
        str: Analysis of the log entry
    """
    # Extract key information
    level = log_entry.get("level", "INFO")
    message = log_entry.get("message", "")
    timestamp = log_entry.get("timestamp", "")

    # Base analysis
    analysis = []

    # Add timestamp analysis
    if timestamp:
        analysis.append(f"Log generated at {timestamp}.")

    # Add severity analysis
    if level in ["ERROR", "CRITICAL"]:
        analysis.append(
            f"This is a {level.lower()} level event that requires immediate attention."
        )
    elif level == "WARNING":
        analysis.append("This is a warning that may require investigation.")

    # Module-specific analysis
    if name == "api":
        if "status" in log_entry:
            status = log_entry["status"]
            if status >= 500:
                analysis.append("Server-side error detected in API response.")
            elif status >= 400:
                analysis.append("Client-side error detected in API request.")
            elif status >= 300:
                analysis.append("API request resulted in a redirection.")
            elif status >= 200:
                analysis.append("API request completed successfully.")

    elif name == "database":
        if "query" in log_entry:
            analysis.append("Database query execution logged.")
            if "duration" in log_entry:
                duration = log_entry["duration"]
                if duration > 1000:
                    analysis.append("Query execution time is unusually high.")

    elif name == "web_server":
        if "method" in log_entry:
            analysis.append(f"Web server processed a {log_entry['method']} request.")
            if "path" in log_entry:
                analysis.append(f"Accessed path: {log_entry['path']}")

    # Add message analysis
    if message:
        analysis.append(f"Message details: {message}")

    # Combine analysis points
    return " ".join(analysis)


def update_progress(name: str, progress: str) -> None:
    """Update progress for a module.

    Args:
        name: Module name
        progress: Progress string to display
    """
    with progress_lock:
        if name not in module_order:
            module_order.append(name)
        module_progress[name] = progress
        module_status[name] = "Running"
        update_progress_display()


def format_json_output(
    success, logs_generated=0, time_taken=0.0, files=None, error=None
):
    """Format output as JSON."""
    if files is None:
        files = []

    output = {
        "success": success,
        "logs_generated": logs_generated,
        "time_taken": time_taken,
        "files": files,
    }

    if error:
        output["error"] = {"message": str(error), "type": error.__class__.__name__}

    return json.dumps(output)


def generate_module_logs(
    module_name: str,
    generator_func: Callable,
    count: int,
    output_file: Union[str, Path],
    llm_format: bool = False,
    json_output: bool = False,
) -> int:
    """Generate logs for a single module.

    Args:
        module_name: Name of the module
        generator_func: Function that generates log entries
        count: Number of log entries to generate
        output_file: Output file path
        llm_format: Whether to generate logs in LLM training format
        json_output: Whether to suppress progress output for JSON mode

    Returns:
        Number of logs generated
    """
    try:
        # Add module to order if not present
        if module_name not in module_order:
            module_order.append(module_name)

        # Set initial status
        with progress_lock:
            module_status[module_name] = "Running"
            if not json_output:
                update_progress_display()

        # Create output directory if needed
        os.makedirs(os.path.dirname(str(output_file)), exist_ok=True)

        # Add file to current run
        current_run_files.add(str(output_file))

        logs_generated = 0
        with open(output_file, "w") as f:
            for _ in range(count):
                if exit_event.is_set():
                    with progress_lock:
                        module_status[module_name] = "Cancelled"
                        if not json_output:
                            update_progress_display()
                    break

                try:
                    log_entry = generator_func()
                    if llm_format:
                        log_entry = generate_llm_format_log(log_entry)
                        f.write(json.dumps(log_entry) + "\n")
                    else:
                        # For non-LLM format, write as plain text if it's a string,
                        # otherwise convert to JSON
                        if isinstance(log_entry, str):
                            f.write(log_entry + "\n")
                        else:
                            f.write(json.dumps(log_entry) + "\n")
                    logs_generated += 1

                    # Update progress every 10%
                    if logs_generated % max(1, count // 10) == 0:
                        with progress_lock:
                            progress = logs_generated / count
                            module_status[
                                module_name
                            ] = f"[{create_progress_bar(progress)}] {int(progress * 100)}%"
                            if not json_output:
                                update_progress_display()
                                print(
                                    f"\033[2K\r{module_name}: {module_status[module_name]}",
                                    end="",
                                    flush=True,
                                )

                except Exception as e:
                    with progress_lock:
                        module_status[module_name] = f"Error: {str(e)}"
                        if not json_output:
                            update_progress_display()
                    raise

        # Update final status
        with progress_lock:
            if not exit_event.is_set():
                module_status[module_name] = "Complete"
            if not json_output:
                update_progress_display()
                print(f"Generated {logs_generated} logs for {module_name}")

        return logs_generated

    except Exception as e:
        with progress_lock:
            module_status[module_name] = f"Error: {str(e)}"
            if not json_output:
                update_progress_display()
        raise


def output_json(result: dict):
    """Format and print JSON output.

    Args:
        result: The result dictionary to output
    """
    # Clean up any ANSI escape sequences
    if "error" in result and isinstance(result["error"].get("message"), str):
        result["error"]["message"] = (
            result["error"]["message"].replace("\033[", "").replace("\x1b[", "")
        )

    # Ensure all fields are present
    if "success" in result and result["success"]:
        result.setdefault("time_taken", 0.0)
        result.setdefault(
            "stats",
            {
                "total_files": len(result.get("files", [])),
                "avg_logs_per_file": (
                    result.get("logs_generated", 0) / len(result.get("files", []))
                    if result.get("files")
                    else 0
                ),
                "total_size_bytes": sum(
                    Path(f).stat().st_size for f in result.get("files", [])
                ),
            },
        )
        result.setdefault(
            "timing",
            {
                "start_time": datetime.now().isoformat(),
                "duration_seconds": result.get("time_taken", 0.0),
                "logs_per_second": (
                    result.get("logs_generated", 0) / result.get("time_taken", 1.0)
                    if result.get("time_taken", 0.0) > 0
                    else 0
                ),
            },
        )
        result.setdefault(
            "config",
            {
                "output_directory": (
                    str(Path(result.get("files", [""])[0]).parent)
                    if result.get("files")
                    else None
                ),
                "file_format": (
                    Path(result.get("files", [""])[0]).suffix[1:]
                    if result.get("files")
                    else None
                ),
            },
        )
    elif "error" in result:
        if isinstance(result["error"], str):
            result["error"] = {"message": result["error"], "type": "str"}
        result.setdefault("logs_generated", 0)
        result.setdefault("time_taken", 0.0)
        result.setdefault("files", [])
        result.setdefault(
            "stats", {"total_files": 0, "avg_logs_per_file": 0, "total_size_bytes": 0}
        )
        result.setdefault("config", {"output_directory": None, "file_format": None})

    # Disable progress display when outputting JSON
    with progress_lock:
        click.echo(json.dumps(result), nl=False)


def generate_llm_format_log(log_entry: Union[str, dict]) -> dict:
    """Format a log entry for LLM training.

    Args:
        log_entry: The log entry to format

    Returns:
        Dictionary containing instruction, input, and output fields
    """
    # Convert string log to dictionary
    if isinstance(log_entry, str):
        log_entry = {"message": log_entry}
        instruction = "Analyze this log message and explain its meaning"
    else:
        # Set instruction based on log level and type
        log_level = log_entry.get("level", "INFO").upper()
        if log_level == "ERROR":
            instruction = "Analyze this error log and suggest potential solutions"
        else:
            instruction = (
                "Analyze this log entry and identify any anomalies or patterns"
            )

    # Generate human-readable analysis
    output_parts = []

    # Add log level if present
    if "level" in log_entry:
        output_parts.append(f"This is a {log_entry['level'].lower()}-level log")

    # Add service info
    if "service" in log_entry:
        output_parts.append(f"from the {log_entry['service']} service")

    # Add type-specific info
    if "type" in log_entry:
        if log_entry.get("service") == "api":
            if log_entry["type"].lower() == "graphql":
                output_parts.append("This is a GraphQL API log")
            elif log_entry["type"].lower() == "rest":
                output_parts.append("This is a REST API log")
        else:
            output_parts.append(
                f"The {log_entry['type']} event indicates: {log_entry.get('message', 'No message provided')}"
            )
    elif "message" in log_entry:
        output_parts.append(f"The event indicates: {log_entry['message']}")

    # Add any additional context
    if "status" in log_entry:
        output_parts.append(f"Status code: {log_entry['status']}")
    if "duration" in log_entry:
        output_parts.append(f"Duration: {log_entry['duration']}ms")
    if "path" in log_entry:
        output_parts.append(f"Path: {log_entry['path']}")
    if "method" in log_entry:
        output_parts.append(f"HTTP Method: {log_entry['method']}")

    # Add timestamp if present
    if "timestamp" in log_entry:
        output_parts.append(f"Timestamp: {log_entry['timestamp']}")

    # Add any additional fields not already included
    for key, value in log_entry.items():
        if key not in {
            "level",
            "service",
            "type",
            "message",
            "status",
            "duration",
            "path",
            "method",
            "timestamp",
        }:
            output_parts.append(f"{key}: {value}")

    # Ensure message is always included in output
    if "message" in log_entry and not any(
        "event indicates" in part for part in output_parts
    ):
        output_parts.append(f"Message: {log_entry['message']}")

    return {
        "instruction": instruction,
        "input": json.dumps(log_entry, indent=2),
        "output": ". ".join(output_parts),
    }


def create_progress_bar(progress: float, width: int = 10) -> str:
    """Create a progress bar string.

    Args:
        progress: Progress value between 0 and 1
        width: Width of the progress bar

    Returns:
        Progress bar string
    """
    filled = int(progress * width)
    empty = width - filled - 1  # Subtract 1 for the '>' character
    return "=" * filled + ">" + " " * empty


class CustomCommand(click.Command):
    """Custom Click command that uses Rich for help display."""

    def get_help(self, ctx: click.Context) -> str:
        """Show help message with Rich formatting."""
        if not hasattr(ctx, "_showing_help"):
            ctx._showing_help = True
            show_rich_help(ctx)
            delattr(ctx, "_showing_help")
            return ""
        return super().get_help(ctx)

    def parse_args(self, ctx: click.Context, args: list) -> list:
        """Parse command line arguments.

        Args:
            ctx: Click context
            args: Command line arguments

        Returns:
            Parsed arguments
        """
        # If using --generate-config, make --config not required
        if "--generate-config" in args:
            for param in self.params:
                if param.name == "config":
                    param.required = False
                    param.type = click.Path(exists=False, dir_okay=False)
        return super().parse_args(ctx, args)

    def invoke(self, ctx: click.Context) -> None:
        """Invoke the command with proper error handling."""
        try:
            return super().invoke(ctx)
        except click.exceptions.Exit as exit_error:
            if exit_error.exit_code == 0:
                sys.exit(0)
            else:
                sys.exit(1)
        except click.exceptions.Abort:
            sys.exit(1)
        except click.UsageError:
            sys.exit(1)
        except Exception:
            sys.exit(1)

    def __call__(self, *args, **kwargs):
        """Override call to handle exit codes."""
        try:
            return super().__call__(*args, **kwargs)
        except SystemExit as e:
            if e.code == 2:  # Convert exit code 2 to 1 for better compatibility
                sys.exit(1)
            raise
        except click.exceptions.Exit as exit_error:
            if exit_error.exit_code == 0:
                sys.exit(0)
            else:
                sys.exit(1)
        except click.exceptions.Abort:
            sys.exit(1)
        except click.UsageError:
            sys.exit(1)
        except Exception:
            sys.exit(1)


@click.command(cls=CustomCommand)
@click.version_option(version=__version__)
@click.option(
    "--generate-config",
    type=click.Path(dir_okay=False),
    help="Generate a full-featured configuration file",
)
@click.option(
    "-c",
    "--count",
    type=click.IntRange(1, 1_000_000),
    default=100,
    help="Number of log entries per module (default: 100, max: 1,000,000)",
)
@click.option(
    "-t",
    "--threads",
    type=click.IntRange(min=1),
    default=os.cpu_count(),
    help="Number of threads to use (default: system CPU count)",
)
@click.option(
    "-f",
    "--config",
    type=click.Path(exists=True, dir_okay=False),
    default="config.json",
    help="Path to config file (default: config.json)",
    required=False,
)
@click.option(
    "-o",
    "--output-dir",
    type=click.Path(file_okay=False),
    default="logs/",
    help="Output directory for log files (default: logs/)",
)
@click.option(
    "--json-output",
    is_flag=True,
    help="Output a single line of JSON with generation results",
)
@click.option(
    "--llm-format",
    is_flag=True,
    help="Generate logs in LLM training format (instruction, input, output). Overrides other options for optimal training.",
)
def cli(
    generate_config: Optional[str],
    count: int,
    threads: int,
    config: str,
    output_dir: str,
    json_output: bool,
    llm_format: bool,
) -> None:
    """Multi-threaded log generator for testing and development.

    Start with: lg3k --generate-config config.json
    Press Ctrl+C to exit gracefully.
    """
    try:
        if generate_config:
            if os.path.exists(generate_config):
                if json_output:
                    result = {
                        "success": False,
                        "error": {
                            "message": f"File {generate_config} already exists",
                            "type": "FileExistsError",
                        },
                    }
                    output_json(result)
                else:
                    if HAS_RICH and console is not None:
                        console.print(
                            f"[red]Error: {generate_config} already exists[/red]"
                        )
                    else:
                        click.echo(f"Error: {generate_config} already exists", err=True)
                sys.exit(1)

            # Generate default config
            with open(generate_config, "w") as f:
                json.dump(get_default_config(), f, indent=4)
            if json_output:
                result = {
                    "success": True,
                    "logs_generated": 0,
                    "time_taken": 0.0,
                    "files": [generate_config],
                }
                output_json(result)
            else:
                if HAS_RICH and console is not None:
                    console.print(
                        f"[green]Generated config file: {generate_config}[/green]"
                    )
                else:
                    click.echo(f"Generated config file: {generate_config}")
            sys.exit(0)

        # Process services
        try:
            result = process_services(
                SimpleNamespace(
                    config=config,
                    count=count,
                    threads=threads,
                    output_dir=output_dir,
                    json=json_output,
                    llm_format=llm_format,
                )
            )

            if json_output:
                if isinstance(result, dict):
                    output_json(result)
                    sys.exit(0 if result.get("success", False) else 1)
                else:
                    output_json(
                        {
                            "success": False,
                            "error": {"message": "Invalid result", "type": "TypeError"},
                        }
                    )
                    sys.exit(1)
            sys.exit(result)

        except KeyboardInterrupt:
            if json_output:
                output_json(
                    {
                        "success": False,
                        "error": {
                            "message": "Operation cancelled by user",
                            "type": "KeyboardInterrupt",
                        },
                    }
                )
            else:
                print("\nOperation cancelled by user")
            sys.exit(1)
        except Exception as e:
            if json_output:
                output_json(
                    {
                        "success": False,
                        "error": {"message": str(e), "type": type(e).__name__},
                    }
                )
            else:
                print(f"Error: {str(e)}")
            sys.exit(1)

    except Exception as e:
        if json_output:
            output_json(
                {
                    "success": False,
                    "error": {"message": str(e), "type": type(e).__name__},
                }
            )
        else:
            print(f"Error: {str(e)}")
        sys.exit(1)


def process_services(args):
    """Process services based on command line arguments."""
    try:
        # Load configuration
        if not args.json:
            print(f"Debug: Loading config from {args.config}")
        config_data = load_config(args.config)
        if not args.json:
            print(f"Debug: Loaded config: {config_data}")

        # Check for active services
        if not config_data.get("services"):
            raise ValueError("No active services in configuration")

        # Create output directory
        if not args.json:
            print(f"Debug: Creating output directory {args.output_dir}")
        os.makedirs(args.output_dir, exist_ok=True)
        if not args.json:
            print(f"Debug: Output directory exists: {os.path.exists(args.output_dir)}")

        # Load modules
        if not args.json:
            print("Debug: Loading modules")
        modules = load_modules()
        if not args.json:
            print(f"Debug: Loaded modules: {list(modules.keys())}")

        # Generate logs
        files = []
        logs_generated = 0
        start_time = time.time()

        for module in config_data["services"]:
            if module not in modules:
                raise ModuleNotFoundError(f"Module {module} not found")

            output_file = os.path.join(
                args.output_dir, f"{module}_{datetime.now().strftime('%Y%m%d_%H%M%S')}"
            )
            output_file += ".jsonl" if args.llm_format else ".log"
            files.append(output_file)
            if not args.json:
                print(f"Debug: Output file is {output_file}")

            # Create parent directory for output file
            os.makedirs(os.path.dirname(output_file), exist_ok=True)
            if not args.json:
                print(
                    f"Debug: Parent directory exists: {os.path.exists(os.path.dirname(output_file))}"
                )

            logs = generate_module_logs(
                module,
                modules[module],
                args.count,
                output_file,
                args.llm_format,
                args.json,
            )
            logs_generated += logs
            if not args.json:
                print(f"Debug: Generated {logs} logs for {module}")

            if not args.json and HAS_RICH and console is not None:
                console.print(f"[green]Generated {logs} logs for {module}[/green]")

        if args.json:
            return {
                "success": True,
                "logs_generated": logs_generated,
                "time_taken": time.time() - start_time,
                "files": files,
            }
        elif HAS_RICH and console is not None:
            console.print(
                f"[green]Successfully generated {logs_generated} logs across {len(files)} files[/green]"
            )

        return 0

    except Exception as e:
        if not args.json:
            print(f"Debug: Error occurred: {str(e)}")
        if args.json:
            return {
                "success": False,
                "error": {"message": str(e), "type": type(e).__name__},
            }
        else:
            if HAS_RICH and console is not None:
                console.print(f"[red]Error: {str(e)}[/red]")
            else:
                print(f"Error: {str(e)}")
            return 1


def main():
    """CLI entry point."""
    cli()
